@@ -1,17 +1,19 @@
 package k8shandler
 
 import (
+	"context"
 	"fmt"
 	"time"
 
-	"github.com/operator-framework/operator-sdk/pkg/sdk"
 	"github.com/sirupsen/logrus"
 	v1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
+	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/apimachinery/pkg/util/wait"
 	"k8s.io/client-go/util/retry"
+	"sigs.k8s.io/controller-runtime/pkg/client"
 
-	api "github.com/openshift/elasticsearch-operator/pkg/apis/elasticsearch/v1"
+	api "github.com/openshift/elasticsearch-operator/pkg/apis/logging/v1"
 	apps "k8s.io/api/apps/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 )
@@ -26,9 +28,11 @@ type statefulSetNode struct {
 	clusterName       string
 	clusterSize       int32
 	priorReplicaCount int32
+
+	client client.Client
 }
 
-func (statefulSetNode *statefulSetNode) populateReference(nodeName string, node api.ElasticsearchNode, cluster *api.Elasticsearch, roleMap map[api.ElasticsearchNodeRole]bool, replicas int32) {
+func (statefulSetNode *statefulSetNode) populateReference(nodeName string, node api.ElasticsearchNode, cluster *api.Elasticsearch, roleMap map[api.ElasticsearchNodeRole]bool, replicas int32, client client.Client) {
 
 	labels := newLabels(cluster.Name, nodeName, roleMap)
 
@@ -51,7 +55,7 @@ func (statefulSetNode *statefulSetNode) populateReference(nodeName string, node
 		Selector: &metav1.LabelSelector{
 			MatchLabels: newLabelSelector(cluster.Name, nodeName, roleMap),
 		},
-		Template: newPodTemplateSpec(nodeName, cluster.Name, cluster.Namespace, node, cluster.Spec.Spec, labels, roleMap),
+		Template: newPodTemplateSpec(nodeName, cluster.Name, cluster.Namespace, node, cluster.Spec.Spec, labels, roleMap, client),
 		UpdateStrategy: apps.StatefulSetUpdateStrategy{
 			Type: apps.RollingUpdateStatefulSetStrategyType,
 			RollingUpdate: &apps.RollingUpdateStatefulSetStrategy{
@@ -65,6 +69,8 @@ func (statefulSetNode *statefulSetNode) populateReference(nodeName string, node
 
 	statefulSetNode.self = statefulSet
 	statefulSetNode.clusterName = cluster.Name
+
+	statefulSetNode.client = client
 }
 
 func (current *statefulSetNode) updateReference(desired NodeTypeInterface) {
@@ -87,7 +93,7 @@ func (node *statefulSetNode) state() api.ElasticsearchNodeStatus {
 	}*/
 
 	// check if the secretHash changed
-	newSecretHash := getSecretDataHash(node.clusterName, node.self.Namespace)
+	newSecretHash := getSecretDataHash(node.clusterName, node.self.Namespace, node.client)
 	if newSecretHash != node.secretHash {
 		rolloutForReload = v1.ConditionTrue
 	}
@@ -107,7 +113,7 @@ func (node *statefulSetNode) name() string {
 
 func (node *statefulSetNode) waitForNodeRejoinCluster() (error, bool) {
 	err := wait.Poll(time.Second*1, time.Second*60, func() (done bool, err error) {
-		clusterSize, getErr := GetClusterNodeCount(node.clusterName, node.self.Namespace)
+		clusterSize, getErr := GetClusterNodeCount(node.clusterName, node.self.Namespace, node.client)
 		if err != nil {
 			logrus.Warnf("Unable to get cluster size waiting for %v to rejoin cluster", node.name())
 			return false, getErr
@@ -121,7 +127,7 @@ func (node *statefulSetNode) waitForNodeRejoinCluster() (error, bool) {
 
 func (node *statefulSetNode) waitForNodeLeaveCluster() (error, bool) {
 	err := wait.Poll(time.Second*1, time.Second*60, func() (done bool, err error) {
-		clusterSize, getErr := GetClusterNodeCount(node.clusterName, node.self.Namespace)
+		clusterSize, getErr := GetClusterNodeCount(node.clusterName, node.self.Namespace, node.client)
 		if err != nil {
 			logrus.Warnf("Unable to get cluster size waiting for %v to leave cluster", node.name())
 			return false, getErr
@@ -137,7 +143,7 @@ func (node *statefulSetNode) setPartition(partitions int32) error {
 	nretries := -1
 	retryErr := retry.RetryOnConflict(retry.DefaultRetry, func() error {
 		nretries++
-		if getErr := sdk.Get(&node.self); getErr != nil {
+		if getErr := node.client.Get(context.TODO(), types.NamespacedName{Name: node.self.Name, Namespace: node.self.Namespace}, &node.self); getErr != nil {
 			logrus.Debugf("Could not get Elasticsearch node resource %v: %v", node.self.Name, getErr)
 			return getErr
 		}
@@ -148,7 +154,7 @@ func (node *statefulSetNode) setPartition(partitions int32) error {
 
 		node.self.Spec.UpdateStrategy.RollingUpdate.Partition = &partitions
 
-		if updateErr := sdk.Update(&node.self); updateErr != nil {
+		if updateErr := node.client.Update(context.TODO(), &node.self); updateErr != nil {
 			logrus.Debugf("Failed to update node resource %v: %v", node.self.Name, updateErr)
 			return updateErr
 		}
@@ -163,9 +169,9 @@ func (node *statefulSetNode) setPartition(partitions int32) error {
 
 func (node *statefulSetNode) partition() (int32, error) {
 
-	desired := node.self.DeepCopy()
+	desired := &apps.StatefulSet{}
 
-	if err := sdk.Get(desired); err != nil {
+	if err := node.client.Get(context.TODO(), types.NamespacedName{Name: node.self.Name, Namespace: node.self.Namespace}, desired); err != nil {
 		logrus.Debugf("Could not get Elasticsearch node resource %v: %v", node.self.Name, err)
 		return -1, err
 	}
@@ -177,7 +183,7 @@ func (node *statefulSetNode) setReplicaCount(replicas int32) error {
 	nretries := -1
 	retryErr := retry.RetryOnConflict(retry.DefaultRetry, func() error {
 		nretries++
-		if getErr := sdk.Get(&node.self); getErr != nil {
+		if getErr := node.client.Get(context.TODO(), types.NamespacedName{Name: node.self.Name, Namespace: node.self.Namespace}, &node.self); getErr != nil {
 			logrus.Debugf("Could not get Elasticsearch node resource %v: %v", node.self.Name, getErr)
 			return getErr
 		}
@@ -188,7 +194,7 @@ func (node *statefulSetNode) setReplicaCount(replicas int32) error {
 
 		node.self.Spec.Replicas = &replicas
 
-		if updateErr := sdk.Update(&node.self); updateErr != nil {
+		if updateErr := node.client.Update(context.TODO(), &node.self); updateErr != nil {
 			logrus.Debugf("Failed to update node resource %v: %v", node.self.Name, updateErr)
 			return updateErr
 		}
@@ -203,9 +209,9 @@ func (node *statefulSetNode) setReplicaCount(replicas int32) error {
 
 func (node *statefulSetNode) replicaCount() (int32, error) {
 
-	desired := node.self.DeepCopy()
+	desired := &apps.StatefulSet{}
 
-	if err := sdk.Get(desired); err != nil {
+	if err := node.client.Get(context.TODO(), types.NamespacedName{Name: node.self.Name, Namespace: node.self.Namespace}, desired); err != nil {
 		logrus.Debugf("Could not get Elasticsearch node resource %v: %v", node.self.Name, err)
 		return -1, err
 	}
@@ -214,8 +220,8 @@ func (node *statefulSetNode) replicaCount() (int32, error) {
 }
 
 func (node *statefulSetNode) isMissing() bool {
-	getNode := node.self.DeepCopy()
-	if getErr := sdk.Get(getNode); getErr != nil {
+	getNode := &apps.StatefulSet{}
+	if getErr := node.client.Get(context.TODO(), types.NamespacedName{Name: node.name(), Namespace: node.self.Namespace}, getNode); getErr != nil {
 		if errors.IsNotFound(getErr) {
 			return true
 		}
@@ -227,12 +233,12 @@ func (node *statefulSetNode) isMissing() bool {
 func (node *statefulSetNode) restart(upgradeStatus *api.ElasticsearchNodeStatus) {
 
 	if upgradeStatus.UpgradeStatus.UnderUpgrade != v1.ConditionTrue {
-		if status, _ := GetClusterHealth(node.clusterName, node.self.Namespace); status != "green" {
+		if status, _ := GetClusterHealthStatus(node.clusterName, node.self.Namespace, node.client); status != "green" {
 			logrus.Infof("Waiting for cluster to be fully recovered before restarting %v: %v / green", node.name(), status)
 			return
 		}
 
-		size, err := GetClusterNodeCount(node.clusterName, node.self.Namespace)
+		size, err := GetClusterNodeCount(node.clusterName, node.self.Namespace, node.client)
 		if err != nil {
 			logrus.Warnf("Unable to get cluster size prior to restart for %v", node.name())
 			return
@@ -282,8 +288,8 @@ func (node *statefulSetNode) restart(upgradeStatus *api.ElasticsearchNodeStatus)
 			}
 
 			// delete the pod
-			if err := DeletePod(podName, node.self.Namespace); err != nil {
-				logrus.Infof("Unable to delete pod %s for restart: %v", podName, err)
+			if err := DeletePod(podName, node.self.Namespace, node.client); err != nil {
+				logrus.Infof("Unable to delete pod %v for restart: %v", podName, err)
 				return
 			}
 
@@ -315,13 +321,13 @@ func (node *statefulSetNode) restart(upgradeStatus *api.ElasticsearchNodeStatus)
 }
 
 func (node *statefulSetNode) delete() {
-	sdk.Delete(&node.self)
+	node.client.Delete(context.TODO(), &node.self)
 }
 
 func (node *statefulSetNode) create() error {
 
 	if node.self.ObjectMeta.ResourceVersion == "" {
-		err := sdk.Create(&node.self)
+		err := node.client.Create(context.TODO(), &node.self)
 		if err != nil {
 			if !errors.IsAlreadyExists(err) {
 				return fmt.Errorf("Could not create node resource: %v", err)
@@ -332,8 +338,8 @@ func (node *statefulSetNode) create() error {
 		}
 
 		// update the hashmaps
-		node.configmapHash = getConfigmapDataHash(node.clusterName, node.self.Namespace)
-		node.secretHash = getSecretDataHash(node.clusterName, node.self.Namespace)
+		node.configmapHash = getConfigmapDataHash(node.clusterName, node.self.Namespace, node.client)
+		node.secretHash = getSecretDataHash(node.clusterName, node.self.Namespace, node.client)
 	} else {
 		node.scale()
 	}
@@ -347,7 +353,7 @@ func (node *statefulSetNode) executeUpdate() error {
 		// isChanged() will get the latest revision from the apiserver
 		// and return false if there is nothing to change and will update the node object if required
 		if node.isChanged() {
-			if updateErr := sdk.Update(&node.self); updateErr != nil {
+			if updateErr := node.client.Update(context.TODO(), &node.self); updateErr != nil {
 				logrus.Debugf("Failed to update node resource %v: %v", node.self.Name, updateErr)
 				return updateErr
 			}
@@ -358,12 +364,12 @@ func (node *statefulSetNode) executeUpdate() error {
 
 func (node *statefulSetNode) update(upgradeStatus *api.ElasticsearchNodeStatus) error {
 	if upgradeStatus.UpgradeStatus.UnderUpgrade != v1.ConditionTrue {
-		if status, _ := GetClusterHealth(node.clusterName, node.self.Namespace); status != "green" {
+		if status, _ := GetClusterHealthStatus(node.clusterName, node.self.Namespace, node.client); status != "green" {
 			logrus.Infof("Waiting for cluster to be fully recovered before restarting %v: %v / green", node.name(), status)
 			return fmt.Errorf("Waiting for cluster to be fully recovered before restarting %v: %v / green", node.name(), status)
 		}
 
-		size, err := GetClusterNodeCount(node.clusterName, node.self.Namespace)
+		size, err := GetClusterNodeCount(node.clusterName, node.self.Namespace, node.client)
 		if err != nil {
 			logrus.Warnf("Unable to get cluster size prior to restart for %v", node.name())
 		}
@@ -439,12 +445,12 @@ func (node *statefulSetNode) update(upgradeStatus *api.ElasticsearchNodeStatus)
 }
 
 func (node *statefulSetNode) refreshHashes() {
-	newConfigmapHash := getConfigmapDataHash(node.clusterName, node.self.Namespace)
+	newConfigmapHash := getConfigmapDataHash(node.clusterName, node.self.Namespace, node.client)
 	if newConfigmapHash != node.configmapHash {
 		node.configmapHash = newConfigmapHash
 	}
 
-	newSecretHash := getSecretDataHash(node.clusterName, node.self.Namespace)
+	newSecretHash := getSecretDataHash(node.clusterName, node.self.Namespace, node.client)
 	if newSecretHash != node.secretHash {
 		node.secretHash = newSecretHash
 	}
@@ -453,7 +459,7 @@ func (node *statefulSetNode) refreshHashes() {
 func (node *statefulSetNode) scale() {
 
 	desired := node.self.DeepCopy()
-	err := sdk.Get(&node.self)
+	err := node.client.Get(context.TODO(), types.NamespacedName{Name: node.self.Name, Namespace: node.self.Namespace}, &node.self)
 	// error check that it exists, etc
 	if err != nil {
 		// if it doesn't exist, return true
@@ -473,11 +479,10 @@ func (node *statefulSetNode) isChanged() bool {
 	changed := false
 
 	desired := node.self.DeepCopy()
-
 	// we want to blank this out before a get to ensure we get the correct information back (possible sdk issue with maps?)
 	node.self.Spec = apps.StatefulSetSpec{}
 
-	err := sdk.Get(&node.self)
+	err := node.client.Get(context.TODO(), types.NamespacedName{Name: node.self.Name, Namespace: node.self.Namespace}, &node.self)
 	// error check that it exists, etc
 	if err != nil {
 		// if it doesn't exist, return true
@@ -491,6 +496,13 @@ func (node *statefulSetNode) isChanged() bool {
 		changed = true
 	}
 
+	// check the pod's tolerations
+	if !areTolerationsSame(node.self.Spec.Template.Spec.Tolerations, desired.Spec.Template.Spec.Tolerations) {
+		logrus.Debugf("Resource '%s' has different tolerations than desired", node.self.Name)
+		node.self.Spec.Template.Spec.Tolerations = desired.Spec.Template.Spec.Tolerations
+		changed = true
+	}
+
 	// Only Image and Resources (CPU & memory) differences trigger rolling restart
 	for index := 0; index < len(node.self.Spec.Template.Spec.Containers); index++ {
 		nodeContainer := node.self.Spec.Template.Spec.Containers[index]
@@ -556,7 +568,7 @@ func (node *statefulSetNode) progressUnshedulableNode(upgradeStatus *api.Elastic
 
 		logrus.Debugf("Updated statefulset %s, manually applying changes on pod: %s", node.name(), podName)
 
-		if err := DeletePod(podName, node.self.Namespace); err != nil {
+		if err := DeletePod(podName, node.self.Namespace, node.client); err != nil {
 			return err
 		}
 
